package org.mercury_im.messenger.ui.chat;

import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;

import io.reactivex.Completable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.functions.Consumer;
import io.reactivex.schedulers.Schedulers;

import org.jxmpp.jid.EntityBareJid;
import org.mercury_im.messenger.MercuryImApplication;
import org.mercury_im.messenger.core.centers.ConnectionCenter;
import org.mercury_im.messenger.core.util.ContactNameUtil;
import org.mercury_im.messenger.persistence.model.ChatModel;
import org.mercury_im.messenger.persistence.model.ContactModel;
import org.mercury_im.messenger.persistence.model.MessageModel;
import org.mercury_im.messenger.persistence.repository.ChatRepository;
import org.mercury_im.messenger.persistence.repository.MessageRepository;
import org.mercury_im.messenger.persistence.repository.RosterRepository;

import java.util.List;

import javax.inject.Inject;

public class ChatViewModel extends ViewModel {

    private final CompositeDisposable disposable = new CompositeDisposable();

    @Inject
    MessageRepository messageRepository;

    @Inject
    RosterRepository rosterRepository;

    @Inject
    ChatRepository chatRepository;

    @Inject
    ConnectionCenter connectionCenter;

    private long accountId;
    private EntityBareJid jid;

    private MutableLiveData<ContactModel> contact = new MutableLiveData<>();
    private MutableLiveData<List<MessageModel>> messages = new MutableLiveData<>();
    private MutableLiveData<String> contactDisplayName = new MutableLiveData<>();
    private MutableLiveData<ChatModel> chat = new MutableLiveData<>();

    public ChatViewModel() {
        super();
        MercuryImApplication.getApplication().getAppComponent().inject(this);
    }

    public void init(long accountId, EntityBareJid jid) {
        this.accountId = accountId;
        this.jid = jid;

        disposable.add(rosterRepository.getContact(accountId, jid)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((Consumer<ContactModel>)
                        contactModel -> {
                            ChatViewModel.this.contact.setValue(contactModel);
                            contactDisplayName.setValue(ContactNameUtil.displayableNameFrom(contactModel));
                        }));

        disposable.add(messageRepository.getAllMessagesOfChat(accountId, jid)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((Consumer<List<MessageModel>>)
                        messages -> ChatViewModel.this.messages.setValue(messages)));

        disposable.add(chatRepository.getOrCreateChatWith(accountId, jid)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((Consumer<ChatModel>)
                        chatModel -> this.chat.setValue(chatModel)));
    }

    @Override
    protected void onCleared() {
        super.onCleared();
        disposable.clear();
    }

    public LiveData<List<MessageModel>> getMessages() {
        return messages;
    }

    public LiveData<ContactModel> getContact() {
        return contact;
    }

    public LiveData<String> getContactDisplayName() {
        return contactDisplayName;
    }

    public void queryTextChanged(String query) {
        if (query.isEmpty()) {
            disposable.add(messageRepository.getAllMessagesOfChat(accountId, jid)
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe((Consumer<List<MessageModel>>)
                            messages -> ChatViewModel.this.messages.setValue(messages)));
        }
        disposable.add(messageRepository.findMessageByQuery(accountId, jid, query)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe((Consumer<List<MessageModel>>) o -> {
                    messages.setValue(o);
                }));
    }

    public Completable requestMamMessages() {
        return Completable.fromAction(() -> {
            ChatModel chatModel = ChatViewModel.this.chat.getValue();
            if (chatModel == null) {
                return;
            }
            connectionCenter.requestMamMessagesFor(chatModel);
        });

    }
}
